/************  Copyright FICOSA International All Rights Reserved  ***********
| Language:          |  MISRA C
| Controller:        |  Independent based on FICOSA Hardware Abstraction Layer
| Spec. Document:    |  none
|------------------------------------------------------------------------------
| Codification Date: | 24/7/2012
|   Date  -  Coder -      Description
| 24/07/12    OC     Creation of the file. 
| 14/08/12    OC     Added the ERR_CTRL_NOTIF callback
| 31/08/12	  OQ	 Remove dual compiler conditional interface (always can handler)
| 27/03/15    FPR	 Adapted to RENESAS RH850F1L
|------------------------------------------------------------------------------
| DESCRIPTION:
|	Wrapper between FicOsekComm and the EB Autosat CAN Driver. 
|   Tested on MPC5604P and MPC5604B/C MCALs
|	Tested on Renesas RH850F1L MCAL
|------------------------------------------------------------------------------
| GENERAL CONSIDERATIONS:
|	The configuration parameters defined in this file are dependent on the
|	corresponding AUTOSAR CAN driver configuration.
|
******************************   END OF HEADER   *****************************/
#ifndef __FICOSARCAN_H
#define __FICOSARCAN_H

/* ------------------------------ Includes ---------------------------------- */

#include "Std_Types.h"
//#include "Can.h"
#include "plib_can1.h"
#include "DiagnosticR/_configurations/FicosarCfg.h"
#include "OsekCom/OsekCom.h"

/*-------------------------------- Defines -------------------------------*/
/* Macros that define in FicosarCfg.h if the CAN peripheral are enabled */
#define CAN_ENABLED   (1)
#define CAN_DISABLED  (0)

/* Macros to set if the driver has to filter automatically all the extended frames or not */
#define CAN_EXTENDED_FRAMES_FILTERED (1)
#define CAN_EXTENDED_FRAMES_NOTIFIED (2)

#define CAN_ID_EXTENDED_MASK  ((UI_32)0x1FFFFFFF)
#define CAN_ID_EXTENDED_FLAG  ((UI_32)0x80000000)

#define CAN_ID_REMOTE_FLAG	  ((UI_32)0x40000000)
#define CAN_ID_EXTENDED_REMOTE_FLAG ((UI_32)0xC0000000)


/* ------------------------------ Data Types -------------------------------- */

/* CAN buffer handler */
typedef UI_8 t_can_buf_hdl; 

/* CAN message */
typedef UI_8 t_can_data[8];

#if (CAN_EXTENDED_FRAMES == CAN_EXTENDED_FRAMES_NOTIFIED)
typedef UI_32 t_idtp;   /* Identifier type in case that the driver */
                        /* works with extended frames */
#else
typedef UI_16 t_idtp;   /* Identifier type in case that the driver */
                        /* works with standard frames */
#endif
 
/* CAN handlers definition */
//typedef enum {
   // CAN1_HANDLER = 0x00,  /*  CAN1 handler */
   // CAN2_HANDLER = 0x01,  /*  CAN2 handler */
   // CAN_HANDLER_UNKNOWN = 0xFF
//}t_can_handler;

/* CAN wrapper logical states*/
typedef enum {
    CAN_STATUS_NOINIT          = 0x00, /* Driver not initialized   */
    CAN_STATUS_ONLINE          = 0x01, /* Driver on line           */
    CAN_STATUS_OFFLINE_PENDING = 0x02, /* Driver going to off line */
    CAN_STATUS_OFFLINE         = 0x03, /* Driver off line          */
    CAN_STATUS_BUSOFF          = 0x04, /* Driver in bus off        */
    CAN_STATUS_SLEEP           = 0x05  /* Driver in sleep mode     */
} t_can_status;

/* CAN buffer object structure for each channel */
//typedef struct {
	//Can_PduType stTxBuffer0; /*Can_PduType elements: id, swPduHandle, length, sdu */
	//BOOL Tx0Nofif;
	//Can_PduType stTxBuffer1;
//	BOOL Tx1Nofif;
//    Can_PduType stRxBuffer;
//} t_Can_BufferObject;

/* ------------------------------ Functions --------------------------------- */

/*****************************************************************************
| Macro:  CAN_CALC_STD_ID
|----------------------------------------------------------------------------
| Operations contract:
|  * Computing idtp macro for a standard ordinary frame. This macro is thought 
|    to be solved during the compiling time to a constant value.
|---------------------------------------------------------------------------
| Parameters explanation:
|   id: is the standard ordinary frame id
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: 0 cpu cycles | O(n): CTE 
| Tmax Int En : 0 cpu cycles | O(n): CTE 
| Tmax Total  : 0 cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
//#define CAN_CALC_STD_ID(id) ((t_idtp)(id))


/*****************************************************************************
| Macro:  CAN_CALC_STD_REMOTE_ID
|----------------------------------------------------------------------------
| Operations contract:
|  * Computing idtp macro for a standard remote frame. This macro is thought 
|    to be solved during the compiling time to a constant value.
|---------------------------------------------------------------------------
| Parameters explanation:
|   id: is the standard remote frame id
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: 0 cpu cycles | O(n): CTE 
| Tmax Int En : 0 cpu cycles | O(n): CTE 
| Tmax Total  : 0 cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
#define CAN_CALC_STD_REMOTE_ID(id) 	((t_idtp)((t_idtp)(id) | CAN_ID_REMOTE_FLAG))


/*****************************************************************************
| Macro:  CAN_CALC_EXT_ID
|----------------------------------------------------------------------------
| Operations contract:
|  * Computing idtp macro for a extended ordinary frame. This macro is thought 
|    to be solved during the compiling time to a constant value.
|---------------------------------------------------------------------------
| Parameters explanation:
|   id: is the extended ordinary frame id
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: 0 cpu cycles | O(n): CTE 
| Tmax Int En : 0 cpu cycles | O(n): CTE 
| Tmax Total  : 0 cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
#if (CAN_EXTENDED_FRAMES == CAN_EXTENDED_FRAMES_NOTIFIED)
	#define CAN_CALC_EXT_ID(id) ((t_idtp)((t_idtp)(id) | CAN_ID_EXTENDED_FLAG))
#endif

/*****************************************************************************
| Macro:  CAN_CALC_EXT_REMOTE_ID
|----------------------------------------------------------------------------
| Operations contract:
|  * Computing idtp macro for a extended remote frame. This macro is thought 
|    to be solved during the compiling time to a constant value.
|---------------------------------------------------------------------------
| Parameters explanation:
|   id: is the extended remote frame id
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: 0 cpu cycles | O(n): CTE 
| Tmax Int En : 0 cpu cycles | O(n): CTE 
| Tmax Total  : 0 cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
#if (CAN_EXTENDED_FRAMES == CAN_EXTENDED_FRAMES_NOTIFIED)
	#define CAN_CALC_EXT_REMOTE_ID(id) ((t_idtp)((t_idtp)(id) | CAN_ID_EXTENDED_REMOTE_FLAG))
#endif


/*---------------------- prototips de funcions ---------------------------*/

/*****************************************************************************
| Routine: CanInit
|----------------------------------------------------------------------------
| Operations contract:
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
//void CanInit(t_can_handler can_handler, const Can_ConfigType *CanConfigType);

/*****************************************************************************
| Routine: CanReinit
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to initialize again the CAN peripheral going from any state to
|    normal mode.
|  * After calling this routine the driver will be in OffLine mode.
|  * Before calling this routine the peripheral must be initialized with the
|    CanInit routine.
|  * Before calling this routine the user must be sure that the peripheral
|    doesn't have any TX pending frame. A way to do that is verifying
|    that the driver is in OffLine mode.
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanReinit(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanSleep
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to put the CAN peripheral in a low consumption mode.
|  * Before calling this routine the peripheral must be initialized with the
|    CanInit routine.
|  * Before calling this routine the user must be sure that the peripheral
|    doesn't have any TX pending frame. A way to do that is verifying
|    that the driver is in OffLine mode.
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanSleep(t_can_handler can_handler);

/*****************************************************************************
| Routine: CanStop
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to put the CAN peripheral into a non initialized state. 
|  * The state after calling this routine will be the same as before calling
|  * the CanInit routine.
|  * Before calling this routine the user must be sure that the peripheral
|    doesn't have any TX pending frame. A way to do that is verifying
|    that the driver is in OffLine mode.
|  * To use the driver again follow the init sequence.
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanStop(t_can_handler can_handler);

/*****************************************************************************
| Routine: CanOnLine
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to open the CAN communications
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanOnLine(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanOffLine
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to close the CAN communications
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanOffLine(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanBufQueryIdtp
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to get a buffer idtp. The idtp identify an unique frame defined
|    by his id and his type.
|---------------------------------------------------------------------------
| Parameters explanation:
|  bhdl: Buffer hardware handler from which the information will be got
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
//t_idtp CanBufQueryIdTp(t_can_handler can_handler, t_can_buf_hdl bhdl);


/*****************************************************************************
| Routine: CanTxRequest
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to ask for a free hardware buffer to transmit a CAN frame.
|  * This routine will active the callback TX mechanism if that wasn't 
|    activated before.
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
void CanTxRequest(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanTx
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to transmit a CAN frame.
|---------------------------------------------------------------------------
| Parameters explanation:
|  notif: Boolean that tells the driver if a transmission notification is
|         required
|         TRUE -> a notification is required
|         FALSE -> no notification requiered
|  idtp: Idtp of the frame that will be transmited
|  len: Data bytes length of the frame
|  can_data: Adress of the variable that have the data to be transmited. If 
|            no data is wanted to be transmited, the user can put a 0 in the
|            len parameter and a NULL in the can_data parameter.
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
//void CanTx(t_can_handler can_handler, BOOL notif, t_idtp idtp, UI_8 len,t_can_data can_data);


/*****************************************************************************
| Routine: CanReadStatus
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to check the current CAN driver state
|---------------------------------------------------------------------------
| Parameters explanation:
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
t_can_status CanReadStatus(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanBufQueryDataLen
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to get the hw buffer status.
|---------------------------------------------------------------------------
| Parameters explanation:
|  can_handler: Identifier of CAN handler
|  result: TRUE  - buffer free
|		   FALSE - buffer full
/---------------------------------------------------------------------------*/
BOOL CanQueryTxBufEmpty(t_can_handler can_handler);


/*****************************************************************************
| Routine: CanBufQueryDataLen
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to get a hardware buffer data length.
|---------------------------------------------------------------------------
| Parameters explanation:
|  bhdl: buffer hardware handler from which the information is required
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/

#define CAN_BUF_QUERY_DATA_LEN(HwCtl, bhdl) (CanBufQueryDataLen(HwCtl, bhdl))
UI_8 CanBufQueryDataLen(t_can_handler can_handler, t_can_buf_hdl bhdl);

/*****************************************************************************
| Routine: CanBufQueryDataByte
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine to get a hardware buffer data bytes.
|---------------------------------------------------------------------------
| Parameters explanation:
|  bhdl: buffer hardware handler from which the information is required
|  byte: data byte index required [0..7]
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: TBD cpu cycles | O(n): CTE 
| Tmax Int En : TBD cpu cycles | O(n): CTE 
| Tmax Total  : TBD cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
//#define CAN_BUF_QUERY_DATA_BYTE(HwCtl, bhdl, byte) (CanBufQueryDataByte(HwCtl, bhdl, byte))
//UI_8 CanBufQueryDataByte(t_can_handler can_handler, t_can_buf_hdl bhdl, UI_8 index);

/*****************************************************************************
| Routine: CanQueryRxActivity
|----------------------------------------------------------------------------
| Operations contract:
|  * Routine that says if there was CAN activity since last check
|---------------------------------------------------------------------------
| Parameters explanation:
|   return: TRUE if has been CAN activity since last check
|---------------------------------------------------------------------------
| Execution time:
| Tmax Int Dis: ??? cpu cycles | O(n): CTE 
| Tmax Int En : ??? cpu cycles | O(n): CTE 
| Tmax Total  : ??? cpu cycles | O(n): CTE 
/---------------------------------------------------------------------------*/
BOOL CanQueryRxActivity(t_can_handler can_handler);

#endif
